==> xl-plugin-manager-22.0.10-sources.jar: com/xebialabs/plugin/manager/rest/api/PluginResourceImpl.scala <==
package com.xebialabs.plugin.manager.rest.api

import com.xebialabs.deployit.core.rest.secured.AbstractSecuredResource
import com.xebialabs.deployit.security.permission.PlatformPermissions.ADMIN
import com.xebialabs.plugin.manager.PluginId.{Artifact, LocalFile}
import com.xebialabs.plugin.manager.metadata.{ArtifactId, PluginMetadata, Version}
import com.xebialabs.plugin.manager.repository.PluginsRepository
import com.xebialabs.plugin.manager.rest.dto.{PluginDto, PluginDtoList, RepositoryDto}
import com.xebialabs.plugin.manager.service.{InstallationResult, PluginService}
import com.xebialabs.plugin.manager.{Plugin, PluginId}
import org.apache.commons.io.IOUtils
import org.jboss.resteasy.plugins.providers.multipart.{InputPart, MultipartFormDataInput}
import org.joda.time.format.DateTimeFormat
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller

import java.io.{InputStream, OutputStream}
import java.util
import java.util.Collections
import javax.ws.rs._
import javax.ws.rs.core.{MediaType, Response, StreamingOutput}
import scala.concurrent.duration._
import scala.concurrent.{Await, ExecutionContext, Future}
import scala.language.postfixOps
import scala.reflect.io.File
import scala.util.{Failure, Success, Try}

@Controller
class PluginResourceImpl @Autowired()(val pluginService: PluginService) extends AbstractSecuredResource with PluginResource {

  val DATE_TIME_FORMAT = "MM-dd-YYYY hh:mm:ss z"
  implicit val ec: ExecutionContext = pluginService.pluginManager.ec

  override def list(): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(pluginService.extend(pluginService.listInstalled())),
      MediaType.APPLICATION_JSON
    ).build()
  }

  override def installOrUpdate(input: MultipartFormDataInput, @QueryParam("pluginId") pluginId: String): Response = {
    def install(pluginId: PluginId, inputParts: util.List[InputPart]): Response = {
      val bytes = getBytesFrom(inputParts)
      pluginService.installOrUpdate(Plugin(pluginId, None, bytes))
      Response.ok().build()
    }

    def convertToLocalAndInstall(pluginId: String, inputParts: util.List[InputPart]): Response = {
      Try(LocalFile(pluginId)) match {
        case Success(localFile) => install(localFile, inputParts)
          Response.ok().build()
        case Failure(exception) => Response
          .status(Response.Status.BAD_REQUEST)
          .entity(ValidationResult(exception.getMessage))
          .build()
      }
    }

    def convertAndInstall(pluginId: String, basename: String, version: String, inputParts: util.List[InputPart]): Response = {
      // first check if the plugin is found in metadata
      val searchResults = pluginService.searchOfficial(basename)
      // if it is, then it's an official plugin
      if (searchResults.size == 1) {
        val artifact = searchResults.head._1.toArtifact
        install(artifact.copy(version = Version.fromString(version).orNull), inputParts)
      } else {
        // if not, check if there is an official plugin with same name already installed in the system
        val installedOfficial = pluginService.listInstalled().collect { case a: Artifact => a }.filter(officialPlugin => officialPlugin.artifactId.equalsIgnoreCase(basename))
        // if there is, then it's an official plugin
        if (installedOfficial.size == 1) {
          val installed = installedOfficial.head
          val idString = installed.repository + ":" + installed.groupId + ":" + pluginId
          install(PluginId.Artifact.fromIdString(idString).get, inputParts)
        } else {
          // if not, then it's a 3rd party plugin
          convertToLocalAndInstall(pluginId, inputParts)
        }
      }
    }

    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    pluginId match {
      case PluginId.patternVersionWithNightly(name, versionWithNightly) =>
        convertAndInstall(pluginId, name, versionWithNightly, inputParts)
      case PluginId.pattern(name, version) =>
        convertAndInstall(pluginId, name, version, inputParts)
      case _ =>
        convertToLocalAndInstall(pluginId, inputParts)
    }
  }

  // install: if LocalFile then just install. If not then search the DB for installed. If found prevent install, if not proceed.
  // update local
  // install official
  // update official

  def getBytesFrom(parts: util.List[InputPart]): Array[Byte] = {
    val inputStream = parts.iterator().next().getBody(classOf[InputStream], null)
    IOUtils.toByteArray(inputStream)
  }


  override def install(input: MultipartFormDataInput, @QueryParam("pluginId") pluginId: String): Response = {
    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    val installationResult = pluginService.install(LocalFile(pluginId), getBytesFrom(inputParts))
    toResponse(installationResult)
  }

  override def update(input: MultipartFormDataInput, @QueryParam("existingPluginName") existingPluginName: String, @QueryParam("newPluginId") newPluginId: String): Response = {
    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    val installationResult = pluginService.update(existingPluginName, LocalFile(newPluginId), getBytesFrom(inputParts))
    toResponse(installationResult)
  }

  private def toResponse(installationResult: InstallationResult): Response = {
    val status = if (installationResult.success) Response.Status.OK else Response.Status.BAD_REQUEST
    Response.status(status)
      .entity(ValidationResult(installationResult.message))
      .build()
  }

  override def uninstall(repositoryId: String,
                         groupId: String,
                         artifactId: String,
                         version: String): Response = {
    checkPermission(ADMIN)

    val pluginSource = PluginSource.withName(repositoryId)
    val optionalVersion = if (version.isEmpty) None else Some(version.split("/")(1))

    pluginService.uninstall(pluginSource, groupId, artifactId, optionalVersion).map { _ =>
      Response.ok.build()
    }.fold(
      err => throw err,
      ok => ok
    )
  }

  override def search(query: String): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(pluginService.search(query).map(toPluginDto).toSeq),
      MediaType.APPLICATION_JSON
    ).build()
  }

  override def listRepositories(): Response = {
    checkPermission(ADMIN)
    Response.ok(RepositoryDto(pluginService.repositories.values.toSeq), MediaType.APPLICATION_JSON).build()
  }

  override def updateRepository(repositoryId: String): Response = {
    checkPermission(ADMIN)
    val lastDownloaded = withRepo(repositoryId)(_.update().map(DateTimeFormat.forPattern(DATE_TIME_FORMAT).print(_)))
    Response.ok(
      Collections.singletonMap("lastDownloaded", lastDownloaded)
    ).build()
  }

  override def listRepository(repositoryId: String): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(withRepo(repositoryId)(_.list()).map(toPluginDto).toSeq),
      MediaType.APPLICATION_JSON
    ).build()
  }

  override def installOrUpdateFromRepository(repositoryId: String,
                                             groupId: String,
                                             artifactId: String,
                                             version: String): Response = {
    checkPermission(ADMIN)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version)
    pluginService.installOrUpdateFromRepository(artifact).map { _ =>
      Response.ok(artifact, MediaType.APPLICATION_JSON).build()
    }.fold(
      err => throw err,
      ok => ok
    )
  }

  override def installFromRepository(repositoryId: String,
                                     groupId: String,
                                     artifactId: String,
                                     version: String): Response = {
    checkPermission(ADMIN)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version)
    val installationResult = pluginService.installFromRepository(artifact)
    toResponse(installationResult)
  }


  override def updateFromRepository(repositoryId: String, groupId: String, artifactId: String): Response = {
    checkPermission(ADMIN)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version = Version.zero.id)
    val installationResult = pluginService.updateFromRepository(artifact)
    toResponse(installationResult)
  }

  override def getLogo(repositoryId: String, groupId: String, artifactId: String): Response = {
    checkPermission(ADMIN)
    pluginService.getLogo(repositoryId, artifactId).map { logoFile =>
      val input = File(logoFile).inputStream()
      val stream = new StreamingOutput {
        override def write(output: OutputStream): Unit = {
          output.write(IOUtils.toByteArray(input))
          input.close()
        }
      }
      Response.ok(stream, MediaType.APPLICATION_OCTET_STREAM).build()
    }.getOrElse {
      throw new NotFoundException(s"Couldn't find logo for '$groupId:$artifactId' in repository '$repositoryId'")
    }
  }

  private def withRepo[T](repositoryId: String, duration: Duration = 30 seconds)(action: PluginsRepository => Future[T]): T = {
    withRepo0(repositoryId) { repository =>
      Await.result(action(repository), duration)
    }
  }

  private def withRepo0[T](repositoryId: String)(action: PluginsRepository => T): T = {
    pluginService.repositories.get(repositoryId) match {
      case None =>
        throw new NotFoundException(s"Unknown plugin repository '$repositoryId'")

      case Some(repository) =>
        action(repository)
    }
  }

  def toPluginDto: ((ArtifactId, Option[PluginMetadata])) => PluginDto = tuple => PluginDto(tuple._1.toPluginId(), tuple._2)

}

==> xl-plugin-manager-23.3.0-511.1415-sources.jar: com/xebialabs/plugin/manager/rest/api/PluginResourceImpl.scala <==
package com.xebialabs.plugin.manager.rest.api

import com.xebialabs.deployit.core.rest.secured.AbstractSecuredResource
import com.xebialabs.deployit.security.permission.PlatformPermissions.ADMIN
import com.xebialabs.plugin.manager.PluginId.{Artifact, LocalFile}
import com.xebialabs.plugin.manager.config.ConfigWrapper
import com.xebialabs.plugin.manager.metadata.{ArtifactId, PluginMetadata, Version}
import com.xebialabs.plugin.manager.repository.PluginsRepository
import com.xebialabs.plugin.manager.rest.dto.{PluginDto, PluginDtoList, RepositoryDto}
import com.xebialabs.plugin.manager.service.{InstallationResult, PluginService}
import com.xebialabs.plugin.manager.{Plugin, PluginId}
import org.apache.commons.io.IOUtils
import org.jboss.resteasy.plugins.providers.multipart.{InputPart, MultipartFormDataInput}
import org.joda.time.format.DateTimeFormat
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller

import java.io.{InputStream, OutputStream}
import java.util
import java.util.Collections
import javax.ws.rs._
import javax.ws.rs.core.{MediaType, Response, StreamingOutput}
import scala.concurrent.duration._
import scala.concurrent.{Await, ExecutionContext, Future}
import scala.language.postfixOps
import scala.reflect.io.File
import scala.util.{Failure, Success, Try}

@Controller
class PluginResourceImpl @Autowired()(val pluginService: PluginService) extends AbstractSecuredResource with PluginResource {

  val DATE_TIME_FORMAT = "MM-dd-YYYY hh:mm:ss z"
  implicit val ec: ExecutionContext = pluginService.pluginManager.ec

  override def list(): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(pluginService.extend(pluginService.listInstalled())),
      MediaType.APPLICATION_JSON
    ).build()
  }

  override def installOrUpdate(input: MultipartFormDataInput, @QueryParam("pluginId") pluginId: String): Response = {
    def install(pluginId: PluginId, inputParts: util.List[InputPart]): Response = {
      val bytes = getBytesFrom(inputParts)
      pluginService.installOrUpdate(Plugin(pluginId, None, bytes))
      Response.ok().build()
    }

    def convertToLocalAndInstall(pluginId: String, inputParts: util.List[InputPart]): Response = {
      Try(LocalFile(pluginId)) match {
        case Success(localFile) => install(localFile, inputParts)
          Response.ok().build()
        case Failure(exception) => Response
          .status(Response.Status.BAD_REQUEST)
          .entity(ValidationResult(exception.getMessage))
          .build()
      }
    }

    def convertAndInstall(pluginId: String, basename: String, version: String, inputParts: util.List[InputPart]): Response = {
      // first check if the plugin is found in metadata
      val searchResults = pluginService.searchOfficial(basename)
      // if it is, then it's an official plugin
      if (searchResults.size == 1) {
        val artifact = searchResults.head._1.toArtifact
        install(artifact.copy(artifactVersion = Version.fromString(version).orNull), inputParts)
      } else {
        // if not, check if there is an official plugin with same name already installed in the system
        val installedOfficial = pluginService.listInstalled().collect { case a: Artifact => a }.filter(officialPlugin => officialPlugin.artifactId.equalsIgnoreCase(basename))
        // if there is, then it's an official plugin
        if (installedOfficial.size == 1) {
          val installed = installedOfficial.head
          val idString = installed.repository + ":" + installed.groupId + ":" + pluginId
          install(PluginId.Artifact.fromIdString(idString).get, inputParts)
        } else {
          // if not, then it's a 3rd party plugin
          convertToLocalAndInstall(pluginId, inputParts)
        }
      }
    }

    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    pluginId match {
      case PluginId.patternVersionWithNightly(name, versionWithNightly) =>
        convertAndInstall(pluginId, name, versionWithNightly, inputParts)
      case PluginId.pattern(name, version) =>
        convertAndInstall(pluginId, name, version, inputParts)
      case _ =>
        convertToLocalAndInstall(pluginId, inputParts)
    }
  }

  def getBytesFrom(parts: util.List[InputPart]): Array[Byte] = {
    val inputStream = parts.iterator().next().getBody(classOf[InputStream], null)
    IOUtils.toByteArray(inputStream)
  }


  override def install(input: MultipartFormDataInput, @QueryParam("pluginId") pluginId: String): Response = {
    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    // TODO: add null check on input parts
    val installationResult = pluginService.install(LocalFile(pluginId), getBytesFrom(inputParts))
    toResponse(installationResult)
  }

  override def update(input: MultipartFormDataInput, @QueryParam("existingPluginName") existingPluginName: String, @QueryParam("newPluginId") newPluginId: String): Response = {
    checkPermission(ADMIN)
    val uploadForm = input.getFormDataMap
    val inputParts = uploadForm.get("file")

    val installationResult = pluginService.update(existingPluginName, LocalFile(newPluginId), getBytesFrom(inputParts))
    toResponse(installationResult)
  }

  private def toResponse(installationResult: InstallationResult): Response = {
    val status = if (installationResult.success) Response.Status.OK else Response.Status.BAD_REQUEST
    Response.status(status)
      .entity(ValidationResult(installationResult.message))
      .build()
  }

  override def uninstall(repositoryId: String,
                         groupId: String,
                         artifactId: String,
                         version: String): Response = {
    checkPermission(ADMIN)

    val pluginSource = PluginSource.withName(repositoryId)
    val optionalVersion = if (version.isEmpty) None else Some(version.split("/")(1))

    pluginService.uninstall(pluginSource, groupId, artifactId, optionalVersion).map { _ =>
      Response.ok.build()
    }.fold(
      err => throw err,
      ok => ok
    )
  }

  override def search(query: String): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(pluginService.search(query).map(toPluginDto).toSeq),
      MediaType.APPLICATION_JSON
    ).build()
  }

  override def listRepositories(): Response = {
    checkPermission(ADMIN)
    Response.ok(RepositoryDto(pluginService.repositories.values.toSeq), MediaType.APPLICATION_JSON).build()
  }

  override def updateRepository(repositoryId: String): Response = {
    checkPermission(ADMIN)
    val lastDownloaded = withRepo(repositoryId)(_.update().map(DateTimeFormat.forPattern(DATE_TIME_FORMAT).print(_)))
    Response.ok(
      Collections.singletonMap("lastDownloaded", lastDownloaded)
    ).build()
  }

  override def listRepository(repositoryId: String): Response = {
    checkPermission(ADMIN)
    Response.ok(
      PluginDtoList(withRepo(repositoryId)(_.list()).map(toPluginDto).toSeq),
      MediaType.APPLICATION_JSON
    ).build()
  }

  // TODO: S-91233 should remove this method, this is just a hack for the time being until we have support for multiple
  //  extensions in official repositories

  private def defaultRepositoryExtension(groupId: String): String = {
    if (groupId == ConfigWrapper.EXTENSION_XLDP) {
      ConfigWrapper.EXTENSION_XLDP
    } else {
      ConfigWrapper.EXTENSION_JAR
    }
  }

  override def installOrUpdateFromRepository(repositoryId: String,
                                             groupId: String,
                                             artifactId: String,
                                             version: String): Response = {
    checkPermission(ADMIN)
    val extension = defaultRepositoryExtension(groupId)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version, extension)
    pluginService.installOrUpdateFromRepository(artifact).map { _ =>
      Response.ok(artifact, MediaType.APPLICATION_JSON).build()
    }.fold(
      err => throw err,
      ok => ok
    )
  }

  override def installFromRepository(repositoryId: String,
                                     groupId: String,
                                     artifactId: String,
                                     version: String): Response = {
    checkPermission(ADMIN)
    val extension = defaultRepositoryExtension(groupId)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version, extension)
    val installationResult = pluginService.installFromRepository(artifact)
    toResponse(installationResult)
  }


  override def updateFromRepository(repositoryId: String, groupId: String, artifactId: String): Response = {
    checkPermission(ADMIN)
    val extension = defaultRepositoryExtension(groupId)
    val artifact = PluginId.Artifact.apply(repositoryId, groupId, artifactId, version = Version.zero.id, extension)
    val installationResult = pluginService.updateFromRepository(artifact)
    toResponse(installationResult)
  }

  override def getLogo(repositoryId: String, groupId: String, artifactId: String): Response = {
    checkPermission(ADMIN)
    pluginService.getLogo(repositoryId, artifactId).map { logoFile =>
      val input = File(logoFile).inputStream()
      val stream = new StreamingOutput {
        override def write(output: OutputStream): Unit = {
          output.write(IOUtils.toByteArray(input))
          input.close()
        }
      }
      Response.ok(stream, MediaType.APPLICATION_OCTET_STREAM).build()
    }.getOrElse {
      throw new NotFoundException(s"Couldn't find logo for '$groupId:$artifactId' in repository '$repositoryId'")
    }
  }

  private def withRepo[T](repositoryId: String, duration: Duration = 30 seconds)(action: PluginsRepository => Future[T]): T = {
    withRepo0(repositoryId) { repository =>
      Await.result(action(repository), duration)
    }
  }

  private def withRepo0[T](repositoryId: String)(action: PluginsRepository => T): T = {
    pluginService.repositories.get(repositoryId) match {
      case None =>
        throw new NotFoundException(s"Unknown plugin repository '$repositoryId'")

      case Some(repository) =>
        action(repository)
    }
  }

  def toPluginDto: ((ArtifactId, Option[PluginMetadata])) => PluginDto = tuple => PluginDto(tuple._1.toPluginId(), tuple._2)

}
